require_relative 'pledge_class'
require_relative 'fundable'

module HoodimusPrime

  class Project
    include Fundable
    
    attr_accessor :name
    attr_reader :balance, :goal
    
    def initialize(name, balance=0, goal=0)
      @name = name.upcase
      @balance = balance
      @goal = goal
      @new_pledges = Hash.new(0)
    end

    def self.from_csv(string)
      name, balance, goal = string.split(',')
      Project.new(name, Integer(balance), Integer(goal))
    end
    
    def to_s
      "Project #{@name} has $#{total_funds} in funding towards a goal of $#{@goal}."
    end

    def <=>(other)
      other.short <=> short
    end

    def new_pledges(pledge)
      @new_pledges[pledge.name] += pledge.amount
      puts "Project #{@name} received a #{pledge.name} pledge worth $#{pledge.amount}."
      puts "Project #{@name}'s pledges: #{@new_pledges}"
    end

    def pledges
      @new_pledges.values.reduce(0, :+)
    end

    def total_funds_short
      @goal - total_funds
    end

    def fully_funded?
      total_funds_short <= 0
    end


    def total_funds
      @balance + pledges
    end

    def each_new_pledge
      @new_pledges.each do |name, amount|
        yield Pledge.new(name, amount)
      end
    end
  end
end